"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var uml = require("../uml/index");
var formatter_1 = require("./formatter");
/**
 * PlantUML Formatter
 *
 * http://plantuml.com
 *
 * @export
 * @class Formatter
 * @extends {AbstractFormatter}
 */
var Formatter = (function (_super) {
    __extends(Formatter, _super);
    function Formatter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Formatter.prototype.generateClassDiagram = function (umlCodeModel) {
        var content;
        content = this._formatNodes(umlCodeModel);
        content += "\n";
        content += this._formatLinks(umlCodeModel);
        if (this.options.plantuml.diagramTags) {
            return "@startuml\n" + content + "\n@enduml\n";
        }
        else {
            return content;
        }
    };
    Formatter.prototype._formatNodes = function (umlCodeModel) {
        var _this = this;
        var content = [];
        umlCodeModel.nodes.forEach(function (key, node) {
            if (node instanceof uml.Class) {
                var properties = _this._formatProperties(node);
                switch (node.stereotype) {
                    case 2 /* Interface */:
                        content.push("interface " + node.identifier + "{\n" + properties + "\n}\n");
                        break;
                    case 1 /* Abstract */:
                        content.push("abstract class " + node.identifier + "{\n" + properties + "\n}\n");
                        break;
                    case 0 /* None */:
                    default:
                        content.push("class " + node.identifier + "{\n" + properties + "\n}\n");
                        break;
                }
            }
        });
        return content.join("\n");
    };
    Formatter.prototype._formatProperties = function (node) {
        var _this = this;
        var properties = [];
        node.variables.forEach(function (key, variable) {
            properties.push(_this._formatVariable(variable));
        });
        node.methods.forEach(function (key, method) {
            properties.push(_this._formatMethod(method));
        });
        return properties.join("\n");
    };
    Formatter.prototype._formatVariable = function (variable) {
        var accessibility = this._formatAccessibility(variable.accessibility);
        var stereotype = "";
        switch (variable.stereotype) {
            case 4 /* Set */:
                stereotype = "<<writeonly>>";
                break;
            case 3 /* Get */:
                stereotype = "<<readonly>>";
                break;
            default:
                break;
        }
        if (stereotype !== "") {
            stereotype = " " + stereotype + " ";
        }
        return "  " + accessibility + stereotype + variable.identifier + " : " + variable.type.text;
    };
    Formatter.prototype._formatMethod = function (method) {
        var accessibility = this._formatAccessibility(method.accessibility);
        var returns = "";
        if (method.returnType) {
            returns = ": " + method.returnType.text;
        }
        var parameters = method.parameters.map(function (p) {
            var type = "";
            var initializer = "";
            var questionMark = "";
            if (p.type) {
                type = ": " + p.type.text;
            }
            if (p.defaultInitializer) {
                initializer = " = " + p.defaultInitializer;
            }
            else if (p.optional) {
                questionMark = "?";
            }
            return "" + p.identifier + questionMark + type + initializer;
        });
        return "  " + accessibility + method.identifier + "(" + parameters.join(", ") + ")" + returns;
    };
    Formatter.prototype._formatLinks = function (umlCodeModel) {
        var content = [];
        umlCodeModel.associations.forEach(function (link) {
            var reverse = link.reverse();
            if (umlCodeModel.associations.contains(reverse)) {
                if (link.toString().localeCompare(reverse.toString()) < 0) {
                    content.push(link.fromName + " -- " + link.toName);
                }
            }
            else {
                content.push(link.fromName + " --> " + link.toName);
            }
        });
        umlCodeModel.generalizations.forEach(function (link) {
            content.push(link.toName + " <|-- " + link.fromName);
        });
        return content.join("\n");
    };
    Formatter.prototype._formatAccessibility = function (accessibility) {
        var result = "";
        switch (accessibility) {
            default:
            case 0 /* Public */:
                result = "+";
                break;
            case 1 /* Protected */:
                result = "#";
                break;
            case 2 /* Private */:
                result = "-";
                break;
        }
        return result;
    };
    return Formatter;
}(formatter_1.AbstractFormatter));
exports.Formatter = Formatter;
